import 'package:flutter/material.dart';
import 'view.dart';

class OwlnextAutocomplete extends StatefulWidget {
  final TextEditingController controller;
  final void Function(String) onChange;
  final void Function(AutocompleteOption) onSelect;
  final List<AutocompleteOption> optionsList;
  final String? label;

  const OwlnextAutocomplete({
    required this.controller,
    required this.onChange,
    required this.optionsList,
    required this.onSelect,
    this.label,
    super.key 
  });

  @override
  OwlnextAutocompleteState createState() {
    return OwlnextAutocompleteState();
  }
}

class OwlnextAutocompleteState extends State<OwlnextAutocomplete> with ViewMixin {
  final OverlayPortalController portalController = OverlayPortalController();
  LayerLink link = LayerLink();
  FocusNode focus = FocusNode();
  double? inputWidth;
  bool isArrowDown = true;

  @override
  void initState() {
    super.initState();
    widget.controller.addListener(onFocusChange);
  }

  @override
  void dispose() {
    super.dispose();
    widget.controller.removeListener(onFocusChange);
  }
  
  void onFocusChange() async {
    setState(() {
      inputWidth = context.size?.width;  
    });

    if (focus.hasFocus == true && portalController.isShowing == false) {
      portalController.show();

      setState(() {
        isArrowDown = false;
      });
    } else if (focus.hasFocus == false) {
      handleClosePortal();
    }
  }

  void handleClosePortal() async {
    portalController.hide();
    setState(() {
      isArrowDown = true;
    });

    await Future.delayed(const Duration(milliseconds: 100));
    focus.unfocus();
  }

  void onSelect(AutocompleteOption selectedOption) {
    widget.onSelect(selectedOption);
    handleClosePortal();
  }

  @override
  Widget build(BuildContext context) {
    return render(context, this);
  }
}

class AutocompleteOption {
  final String label;
  final String value;

  AutocompleteOption({
    required this.label,
    required this.value,
  });
}